package contracts

import (
	"code.wuban.net.cn/service/pancakeswapper/contracts/v3pool"
	"context"
	"github.com/ethereum/go-ethereum/accounts/abi/bind"
	"github.com/ethereum/go-ethereum/common"
	"github.com/ethereum/go-ethereum/ethclient"
	"math/big"
	"testing"
)

func TestGetPrice(t *testing.T) {
	client, _ := ethclient.Dial("https://bscnode.bitheart.org")
	user := common.HexToAddress("0x3e3b4d16ce35840c28f90edb7f38e5bdd976c7e3")
	pool := common.HexToAddress("0xcB0AE3B8337f0Cc35563e6b9fC357F2298C0D24a")
	contract, err := v3pool.NewV3Pool(pool, client)
	if err != nil {
		t.Fatal(err)
	}
	callOpts := &bind.CallOpts{
		BlockNumber: nil,
		From:        user,
		Context:     context.Background(),
	}
	info, err := contract.Slot0(callOpts)
	if err != nil {
		t.Fatal(err)
	}
	t.Log("sqrtPriceX96 is", info.SqrtPriceX96.String())

	// 获取slot0数据，包含价格sqrtPriceX96和tick
	var slot0 = info
	var sqrtPriceX96 = slot0.SqrtPriceX96
	//var tick = slot0.Tick

	priceX96 := new(big.Int).Mul(sqrtPriceX96, sqrtPriceX96)
	//priceX96Dec := priceX96
	dec := big.NewInt(1e18)
	priceX96Dec := new(big.Int).Mul(priceX96, dec)
	price := priceX96Dec.Div(priceX96Dec, new(big.Int).Exp(big.NewInt(2), big.NewInt(192), nil))
	bf := new(big.Float).SetInt(price)
	priceFloat, _ := bf.Float64()
	priceFloat = priceFloat / 1e18
	t.Log("price is", priceFloat)
}
