package core

import (
	"code.wuban.net.cn/service/pancakeswapper/clientpool"
	"code.wuban.net.cn/service/pancakeswapper/config"
	"code.wuban.net.cn/service/pancakeswapper/grouptask"
	"code.wuban.net.cn/service/pancakeswapper/types"
	"code.wuban.net.cn/service/pancakeswapper/work"
	"fmt"
	"time"
)

type Swapper struct {
	cfg *config.Config
}

func NewSwapper(conf *config.Config) (*Swapper, error) {
	n := new(Swapper)
	n.cfg = conf
	return n, nil
}

func (s *Swapper) Run() {
	env := types.GetEnv(s.cfg.Rpc.Env)
	clientpool.InitPool(s.cfg.Rpc.PoolSize, s.cfg.Rpc.Url, s.cfg.Rpc.Proxy)
	work.WorkInit(env.ChainId)

	// 1. read account info from json file
	accs := work.LoadAccountsFromCsv(s.cfg.Users)
	work.SetTotalUser(len(accs))
	waitCh := make(chan struct{})

	// 2. init account info
	go func(w chan struct{}, accounts []*types.Account) {
		grouptask.DoMultiTasks(s.cfg.Param.Routine, env, work.HandlerAccountSwap, accounts)
		//grouptask.DoMultiTasks(s.cfg.Param.Routine, env, work.HandlerAccountSwapWithUniversalRouter, accounts)
		w <- struct{}{}
	}(waitCh, accs)

	tm := time.NewTicker(time.Second * time.Duration(2))
	defer tm.Stop()
	for {
		select {
		case <-tm.C:
			// print progress
			fmt.Println(work.GetReport())
			fmt.Println("")
		case <-waitCh:
			failed := work.Finish()
			if failed > 0 {
				fmt.Printf("all done, %d failed user info write in failed.json\n", failed)
			} else {
				fmt.Println(work.GetReport())
				fmt.Println("all done, no failed user")
			}
			return
		}
	}
}
