package grouptask

import (
	"code.wuban.net.cn/service/pancakeswapper/types"
	"errors"
	"sync"
)

var (
	ErrTaskPoolIsFull = errors.New("task pool is full")
)

type TaskHandle func(acc *types.Account, env types.Env, idx uint)

type Tasks struct {
	tasknum  uint
	handler  TaskHandle
	taskpool chan *types.Account
	env      types.Env
	wg       sync.WaitGroup
}

func NewTasks(routine uint, env types.Env, handle TaskHandle) *Tasks {
	return &Tasks{
		tasknum:  routine,
		handler:  handle,
		env:      env,
		taskpool: make(chan *types.Account, 1000000),
	}
}

func (t *Tasks) AddTask(task *types.Account) error {
	select {
	case t.taskpool <- task:
		return nil
	default:
		return ErrTaskPoolIsFull
	}
}

func (t *Tasks) WaitFinish() {
	close(t.taskpool)
	t.wg.Wait()
}

func (t *Tasks) Run() {
	for i := uint(0); i < t.tasknum; i++ {
		t.wg.Add(1)
		go func(chanIdx uint) {
			defer t.wg.Done()
			for {
				select {
				case task, ok := <-t.taskpool:
					if !ok {
						return
					}
					t.handler(task, t.env, chanIdx)
				}
			}
		}(i)
	}
}

func DoMultiTasks(routine int, env types.Env, handler TaskHandle, items []*types.Account) {
	tasks := NewTasks(uint(routine), env, handler)
	tasks.Run()
	for _, item := range items {
		tasks.AddTask(item)
	}
	tasks.WaitFinish()
}
