package pancake

import (
	ethcommon "github.com/ethereum/go-ethereum/common"
	"math/big"
)

type Commands byte

const (
	V3_SWAP_EXACT_IN      Commands = 0x00
	V3_SWAP_EXACT_OUT     Commands = 0x01
	PERMIT2_TRANSFER_FROM Commands = 0x02
	PERMIT2_PERMIT_BATCH  Commands = 0x03
	SWEEP                 Commands = 0x04
	TRANSFER              Commands = 0x05
	PAY_PORTION           Commands = 0x06

	V2_SWAP_EXACT_IN            Commands = 0x08
	V2_SWAP_EXACT_OUT           Commands = 0x09
	PERMIT2_PERMIT              Commands = 0x0a
	WRAP_ETH                    Commands = 0x0b
	UNWRAP_WETH                 Commands = 0x0c
	PERMIT2_TRANSFER_FROM_BATCH Commands = 0x0d
	BALANCE_CHECK_ERC20         Commands = 0x0e

	INFI_SWAP                  Commands = 0x10
	V3_POSITION_MANAGER_PERMIT Commands = 0x11
	V3_POSITION_MANAGER_CALL   Commands = 0x12
	INFI_CL_INITIALIZE_POOL    Commands = 0x13
	INFI_BIN_INITIALIZE_POOL   Commands = 0x14
	INFI_CL_POSITION_CALL      Commands = 0x15
	INFI_BIN_POSITION_CALL     Commands = 0x16
	// COMMAND_PLACEHOLDER = 0x17 -> 0x20

	// Command Types where 0x21<=value<=0x3f
	EXECUTE_SUB_PLAN      Commands = 0x21
	STABLE_SWAP_EXACT_IN  Commands = 0x22
	STABLE_SWAP_EXACT_OUT Commands = 0x23
)

type CallParam struct {
	Cmd   Commands
	Param interface{}
}

type ParamV3SwapExactIn struct {
	Recipient    ethcommon.Address
	AmountIn     *big.Int
	AmountOutMin *big.Int
	Path         []byte
	PayerIsUser  bool
}
